﻿// -----------------------------------------------------------------------
// <copyright file="CustomDataGrid.cs" company="Microsoft">
// TODO: Update copyright text.
// </copyright>
// -----------------------------------------------------------------------

namespace CustomCalculator.Controls
{
    using Infragistics.Windows.DataPresenter;
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.IO;
    using System.Linq;
    using System.Text;
    using System.Xml;
    using System.Windows;
    using System.Reflection;
    
    /// <summary>
    /// TODO: Update summary.
    /// </summary>
    public class CustomDataGrid : XamDataGrid, INotifyPropertyChanged
    {
        #region 

        private bool? allMemberChecked = false;
        private bool Update = true;
        private string savefile = @"c:/DataGridTesting/SummaryCountTesting.txt";
        private string gridSettings;

        public static readonly DependencyProperty MultiGridProperty = DependencyProperty.Register(
            "MultiGrid", typeof(bool), typeof(CustomDataGrid),
            new FrameworkPropertyMetadata(false, FrameworkPropertyMetadataOptions.AffectsMeasure));

        public bool MultiGrid
        {
            get { return (bool)GetValue(MultiGridProperty); }
            set { this.SetValue(MultiGridProperty, value); }
        }

        public bool? AllMembersAreChecked
        {
            get { return allMemberChecked; }
            set
            {
                if (value == true)
                {
                    if (Update == true)
                    {
                        IEnumerable<DataRecord> filteredInRecords = this.RecordManager.GetFilteredInDataRecords();
                        List<Record> list = new List<Record>(filteredInRecords);
                        this.SelectedItems.Records.AddRange(list.ToArray());
                    }
                }
                if (value == false)
                {
                    this.SelectedItems.Records.Clear();
                }
                allMemberChecked = value;
                this.OnPropertyChanged("AllMembersAreChecked");
            }
        }
        #endregion

        public CustomDataGrid()
        {
            this.SelectedItemsChanged += new EventHandler<Infragistics.Windows.DataPresenter.Events.SelectedItemsChangedEventArgs>(CustomDataGrid_SelectedItemsChanged);
            this.FieldLayoutInitialized += new EventHandler<Infragistics.Windows.DataPresenter.Events.FieldLayoutInitializedEventArgs>(CustomDataGrid_FieldLayoutInitialized);
            this.Loaded += new RoutedEventHandler(CustomDataGrid_Loaded);

            if (Application.Current != null) 
            {
                Application.Current.Exit += new ExitEventHandler(this.Current_Exit);
            }
        }

        #region Events

        public event PropertyChangedEventHandler PropertyChanged;

        protected void OnPropertyChanged(string propertyName)
        {
            this.OnPropertyChanged(new PropertyChangedEventArgs(propertyName));
        }

        protected void OnPropertyChanged(PropertyChangedEventArgs e)
        {
            var handler = this.PropertyChanged;
            if (handler != null)
            {
                handler(this, e);
            }
        }

        void CustomDataGrid_SelectedItemsChanged(object sender, Infragistics.Windows.DataPresenter.Events.SelectedItemsChangedEventArgs e)
        {
            if (this.MultiGrid == true)
            {
                IEnumerable<DataRecord> filteredInRecords = this.RecordManager.GetFilteredInDataRecords();
                List<DataRecord> list = new List<DataRecord>(filteredInRecords);
                Update = true;

                if (AllMembersAreChecked == true || AllMembersAreChecked == false)
                {
                    AllMembersAreChecked = null;
                }

                if (AllMembersAreChecked == null && this.SelectedItems.Records.Count == 0)
                {
                    AllMembersAreChecked = false;
                }

                if (AllMembersAreChecked == null && this.SelectedItems.Records.Count == list.Count)
                {
                    Update = false;
                    AllMembersAreChecked = true;
                }
                            
                // Refresh the custom summary calculators
                SummaryDefinition sumR = (from def in this.FieldLayouts[0].SummaryDefinitions
                                            where def.Calculator is CustomSummaryCalculator
                                            select def).FirstOrDefault();
                if (sumR != null)
                    sumR.Refresh();
            }
        }

        void CustomDataGrid_FieldLayoutInitialized(object sender, Infragistics.Windows.DataPresenter.Events.FieldLayoutInitializedEventArgs e)
        {
            if (this.MultiGrid == true)
            {
                UnboundField CheckboxField = new UnboundField()
                {
                    Name = "checkbox",
                    Label = string.Empty,
                    Width = FieldLength.Auto,
                    FixedLocation = FixedFieldLocation.FixedToNearEdge,
                };

                CheckboxField.Settings.AllowFixing = AllowFieldFixing.No;
                CheckboxField.Settings.AllowHiding = AllowFieldHiding.Never;
                CheckboxField.Settings.AllowRecordFiltering = false;
                CheckboxField.Settings.AllowResize = false;
                CheckboxField.Settings.LabelClickAction = LabelClickAction.Nothing;
                CheckboxField.Settings.LabelPresenterStyle = (Style)this.TryFindResource("CheckAll");
                CheckboxField.Settings.CellValuePresenterStyle = (Style)this.TryFindResource("ShowInChartCellStyle");
                e.FieldLayout.Fields.Insert(0, CheckboxField);
                e.FieldLayout.Settings.FixedFieldUIType = FixedFieldUIType.None;

                CustomSummaryCalculator myCalculater = new CustomSummaryCalculator();
                SummaryCalculator.Register(myCalculater);

                if (this.FieldLayouts[0].SummaryDefinitions != null)
                {
                    SummaryDefinition mySelectedCountDefinition = new SummaryDefinition();
                    mySelectedCountDefinition.SourceFieldName = "checkbox";
                    mySelectedCountDefinition.DisplayArea = SummaryDisplayAreas.BottomFixed;
                    mySelectedCountDefinition.Calculator = myCalculater;

                    this.FieldLayouts[0].SummaryDefinitions.Add(mySelectedCountDefinition);
                }
            }
        }

        void CustomDataGrid_Loaded(object sender, RoutedEventArgs e)
        {
            FileStream myStream = new FileStream(savefile, FileMode.OpenOrCreate);
            this.LoadCustomizations(myStream);


            //*********************************************************************************
            //This is the code you can use to create an XML document in order to read it easily
            //and make the check you need.
            MemoryStream memStream = new MemoryStream();
            using (FileStream fileStream = File.OpenRead(savefile))
            {
                memStream.SetLength(fileStream.Length);
                fileStream.Read(memStream.GetBuffer(), 0, (int)fileStream.Length);
            }
            XmlDocument xDocument = new XmlDocument();
            xDocument.Load(memStream);
            //*********************************************************************************   
        }
                
        void Current_Exit(object sender, ExitEventArgs e)
        {
            FileStream myStream = new FileStream(savefile, FileMode.OpenOrCreate);
            this.SaveCustomizations(myStream);
        }
        #endregion
    }
}
